@@ -16,20 +16,29 @@ from heatclient.openstack.common._i18n import _
 import yaml
 
 
-SECTIONS = (PARAMETER_DEFAULTS, PARAMETERS, RESOURCE_REGISTRY) = \
-           ('parameter_defaults', 'parameters', 'resource_registry')
+SECTIONS = (
+    PARAMETER_DEFAULTS, PARAMETERS, RESOURCE_REGISTRY, EVENT_SINKS
+) = (
+    'parameter_defaults', 'parameters', 'resource_registry', 'event_sinks'
+)
 
 
 def parse(env_str):
-    '''Takes a string and returns a dict containing the parsed structure.
+    """Takes a string and returns a dict containing the parsed structure.
 
     This includes determination of whether the string is using the
     YAML format.
-    '''
+    """
     try:
         env = yaml.load(env_str, Loader=template_format.yaml_loader)
-    except yaml.YAMLError as yea:
-        raise ValueError(yea)
+    except yaml.YAMLError:
+        # NOTE(prazumovsky): we need to return more informative error for
+        # user, so use SafeLoader, which return error message with template
+        # snippet where error has been occurred.
+        try:
+            env = yaml.load(env_str, Loader=yaml.SafeLoader)
+        except yaml.YAMLError as yea:
+            raise ValueError(yea)
     else:
         if env is None:
             env = {}
@@ -45,8 +54,8 @@ def parse(env_str):
 
 
 def default_for_missing(env):
-    '''Checks a parsed environment for missing sections.
-    '''
+    """Checks a parsed environment for missing sections."""
+
     for param in SECTIONS:
         if param not in env:
             env[param] = {}
