@@ -52,6 +52,28 @@ class ShellEnvironmentTest(testtools.TestCase):
         if url:
             self.assertEqual(content.decode('utf-8'), files[url])
 
+    def test_ignore_env_keys(self):
+        self.m.StubOutWithMock(request, 'urlopen')
+        env_file = '/home/my/dir/env.yaml'
+        env = b'''
+        resource_registry:
+          resources:
+            bar:
+              hooks: pre_create
+              restricted_actions: replace
+        '''
+        request.urlopen('file://%s' % env_file).AndReturn(
+            six.BytesIO(env))
+        self.m.ReplayAll()
+        _, env_dict = template_utils.process_environment_and_files(
+            env_file)
+        self.assertEqual(
+            {u'resource_registry': {u'resources': {
+                u'bar': {u'hooks': u'pre_create',
+                         u'restricted_actions': u'replace'}}}},
+            env_dict)
+        self.m.VerifyAll()
+
     def test_process_environment_file(self):
 
         self.m.StubOutWithMock(request, 'urlopen')
@@ -313,6 +335,123 @@ class ShellEnvironmentTest(testtools.TestCase):
         self.assertEqual({}, env)
         self.assertEqual({}, files)
 
+    def test_process_multiple_environments_and_files_from_object(self):
+
+        env_object = 'http://no.where/path/to/env.yaml'
+        env1 = b'''
+        parameters:
+          "param1": "value1"
+        resource_registry:
+          "OS::Thingy1": "b/a.yaml"
+        '''
+
+        self.m.ReplayAll()
+
+        self.object_requested = False
+
+        def env_path_is_object(object_url):
+            return True
+
+        def object_request(method, object_url):
+            self.object_requested = True
+            self.assertEqual('GET', method)
+            self.assertTrue(object_url.startswith("http://no.where/path/to/"))
+            if object_url == env_object:
+                return env1
+            else:
+                return self.template_a
+
+        files, env = template_utils.process_multiple_environments_and_files(
+            env_paths=[env_object], env_path_is_object=env_path_is_object,
+            object_request=object_request)
+        self.assertEqual(
+            {
+                'resource_registry': {
+                    'OS::Thingy1': 'http://no.where/path/to/b/a.yaml'},
+                'parameters': {'param1': 'value1'}
+            },
+            env)
+        self.assertEqual(self.template_a.decode('utf-8'),
+                         files['http://no.where/path/to/b/a.yaml'])
+
+    def test_process_multiple_environments_and_files_tracker(self):
+        # Setup
+        self.m.StubOutWithMock(request, 'urlopen')
+        env_file1 = '/home/my/dir/env1.yaml'
+
+        env1 = b'''
+        parameters:
+          "param1": "value1"
+        resource_registry:
+          "OS::Thingy1": "file:///home/b/a.yaml"
+        '''
+        request.urlopen('file://%s' % env_file1).AndReturn(
+            six.BytesIO(env1))
+        request.urlopen('file:///home/b/a.yaml').AndReturn(
+            six.BytesIO(self.template_a))
+        request.urlopen('file:///home/b/a.yaml').AndReturn(
+            six.BytesIO(self.template_a))
+        self.m.ReplayAll()
+
+        # Test
+        env_file_list = []
+        files, env = template_utils.process_multiple_environments_and_files(
+            [env_file1], env_list_tracker=env_file_list)
+
+        # Verify
+        expected_env = {'parameters': {'param1': 'value1'},
+                        'resource_registry':
+                            {'OS::Thingy1': 'file:///home/b/a.yaml'}
+                        }
+        self.assertEqual(expected_env, env)
+
+        self.assertEqual(self.template_a.decode('utf-8'),
+                         files['file:///home/b/a.yaml'])
+
+        self.assertEqual(['file:///home/my/dir/env1.yaml'], env_file_list)
+        self.assertEqual(json.dumps(expected_env),
+                         files['file:///home/my/dir/env1.yaml'])
+
+    def test_process_environment_relative_file_tracker(self):
+
+        self.m.StubOutWithMock(request, 'urlopen')
+        env_file = '/home/my/dir/env.yaml'
+        env_url = 'file:///home/my/dir/env.yaml'
+        env = b'''
+        resource_registry:
+          "OS::Thingy": a.yaml
+        '''
+
+        request.urlopen(env_url).AndReturn(
+            six.BytesIO(env))
+        request.urlopen('file:///home/my/dir/a.yaml').AndReturn(
+            six.BytesIO(self.template_a))
+        request.urlopen('file:///home/my/dir/a.yaml').AndReturn(
+            six.BytesIO(self.template_a))
+        self.m.ReplayAll()
+
+        self.assertEqual(
+            env_url,
+            utils.normalise_file_path_to_url(env_file))
+        self.assertEqual(
+            'file:///home/my/dir',
+            utils.base_url_for_url(env_url))
+
+        env_file_list = []
+        files, env = template_utils.process_multiple_environments_and_files(
+            [env_file], env_list_tracker=env_file_list)
+
+        # Verify
+        expected_env = {'resource_registry':
+                        {'OS::Thingy': 'file:///home/my/dir/a.yaml'}}
+        self.assertEqual(expected_env, env)
+
+        self.assertEqual(self.template_a.decode('utf-8'),
+                         files['file:///home/my/dir/a.yaml'])
+        self.assertEqual(['file:///home/my/dir/env.yaml'], env_file_list)
+        self.assertEqual(json.dumps(expected_env),
+                         files['file:///home/my/dir/env.yaml'])
+
     def test_global_files(self):
         url = 'file:///home/b/a.yaml'
         env = '''
@@ -405,8 +544,8 @@ class TestGetTemplateContents(testtools.TestCase):
 
     def test_get_template_contents_file(self):
         with tempfile.NamedTemporaryFile() as tmpl_file:
-            tmpl = b'{"AWSTemplateFormatVersion" : "2010-09-09",' \
-                   b' "foo": "bar"}'
+            tmpl = (b'{"AWSTemplateFormatVersion" : "2010-09-09",'
+                    b' "foo": "bar"}')
             tmpl_file.write(tmpl)
             tmpl_file.flush()
 
@@ -439,7 +578,7 @@ class TestGetTemplateContents(testtools.TestCase):
     def test_get_template_contents_file_none_existing(self):
         files, tmpl_parsed = template_utils.get_template_contents(
             existing=True)
-        self.assertEqual(None, tmpl_parsed)
+        self.assertIsNone(tmpl_parsed)
         self.assertEqual({}, files)
 
     def test_get_template_contents_parse_error(self):
@@ -493,6 +632,33 @@ class TestGetTemplateContents(testtools.TestCase):
         self.assertEqual({}, files)
         self.assertTrue(self.object_requested)
 
+    def test_get_nested_stack_template_contents_object(self):
+        tmpl = ('{"heat_template_version": "2016-04-08",'
+                '"resources": {'
+                '"FooBar": {'
+                '"type": "foo/bar.yaml"}}}')
+        url = 'http://no.where/path/to/a.yaml'
+        self.m.ReplayAll()
+
+        self.object_requested = False
+
+        def object_request(method, object_url):
+            self.object_requested = True
+            self.assertEqual('GET', method)
+            self.assertTrue(object_url.startswith("http://no.where/path/to/"))
+            if object_url == url:
+                return tmpl
+            else:
+                return '{"heat_template_version": "2016-04-08"}'
+
+        files, tmpl_parsed = template_utils.get_template_contents(
+            template_object=url,
+            object_request=object_request)
+
+        self.assertEqual(files['http://no.where/path/to/foo/bar.yaml'],
+                         '{"heat_template_version": "2016-04-08"}')
+        self.assertTrue(self.object_requested)
+
     def check_non_utf8_content(self, filename, content):
         base_url = 'file:///tmp'
         url = '%s/%s' % (base_url, filename)
@@ -759,10 +925,10 @@ parameters:
         files, tmpl_parsed = template_utils.get_template_contents(
             template_file=tmpl_file)
 
-        self.assertEqual(yaml.load(self.foo_template.decode('utf-8')),
+        self.assertEqual(yaml.safe_load(self.foo_template.decode('utf-8')),
                          json.loads(files.get('file:///home/my/dir/foo.yaml')))
         self.assertEqual(
-            yaml.load(self.egg_template.decode('utf-8')),
+            yaml.safe_load(self.egg_template.decode('utf-8')),
             json.loads(files.get('file:///home/my/dir/spam/egg.yaml')))
 
         self.assertEqual({
@@ -841,7 +1007,7 @@ parameters:
         files, tmpl_parsed = template_utils.get_template_contents(
             template_file=tmpl_file)
 
-        self.assertEqual(yaml.load(self.bar_template.decode('utf-8')),
+        self.assertEqual(yaml.safe_load(self.bar_template.decode('utf-8')),
                          json.loads(files.get('file:///home/my/dir/bar.yaml')))
 
         self.assertEqual({
@@ -995,7 +1161,7 @@ parameters:
             }
         }, json.loads(files.get(template_url)))
 
-        self.assertEqual(yaml.load(self.foo_template.decode('utf-8')),
+        self.assertEqual(yaml.safe_load(self.foo_template.decode('utf-8')),
                          json.loads(files.get(foo_url)))
         self.assertEqual({
             u'heat_template_version': u'2013-05-23',
@@ -1015,9 +1181,9 @@ parameters:
         }, json.loads(files.get(egg_url)))
         self.assertEqual(b'ham contents',
                          files.get(ham_url))
-        self.assertEqual(yaml.load(self.foo_template.decode('utf-8')),
+        self.assertEqual(yaml.safe_load(self.foo_template.decode('utf-8')),
                          json.loads(files.get(one_url)))
-        self.assertEqual(yaml.load(self.foo_template.decode('utf-8')),
+        self.assertEqual(yaml.safe_load(self.foo_template.decode('utf-8')),
                          json.loads(files.get(two_url)))
         self.assertEqual(b'three contents',
                          files.get(three_url))
