@@ -51,13 +51,19 @@ class ResourceManager(stacks.StackChildManager):
 
     def list(self, stack_id, **kwargs):
         """Get a list of resources.
+
         :rtype: list of :class:`Resource`
         """
         params = {}
 
+        if 'filters' in kwargs:
+            filters = kwargs.pop('filters')
+            params.update(filters)
+
         for key, value in six.iteritems(kwargs):
             if value:
                 params[key] = value
+
         url = '/stacks/%s/resources' % stack_id
         if params:
             url += '?%s' % parse.urlencode(params, True)
@@ -111,10 +117,30 @@ class ResourceManager(stacks.StackChildManager):
         body = utils.get_response_body(resp)
         return body
 
-    def generate_template(self, resource_name):
-        """DEPRECATED! Use `generate_template` of `ResourceTypeManager`
-        instead.
+    def mark_unhealthy(self, stack_id, resource_name,
+                       mark_unhealthy, resource_status_reason):
+        """Mark a resource as healthy or unhealthy.
+
+        :param stack_id: ID of stack containing the resource
+        :param resource_name: ID of resource
+        :param mark_unhealthy: Mark resource unhealthy if set to True
+        :param resource_status_reason: Reason for resource status change.
         """
+        stack_id = self._resolve_stack_id(stack_id)
+        url_str = '/stacks/%s/resources/%s' % (
+                  parse.quote(stack_id, ''),
+                  parse.quote(encodeutils.safe_encode(resource_name), ''))
+        resp = self.client.patch(
+            url_str,
+            data={"mark_unhealthy": mark_unhealthy,
+                  "resource_status_reason": resource_status_reason})
+
+        body = utils.get_response_body(resp)
+        return body
+
+    def generate_template(self, resource_name):
+        """Deprecated in favor of generate_template in ResourceTypeManager."""
+
         url_str = '/resource_types/%s/template' % (
                   parse.quote(encodeutils.safe_encode(resource_name), ''))
         resp = self.client.get(url_str)
